local L = AceLibrary("AceLocale-2.2"):new("fmBuffCheck")

fmBuffCheck = AceLibrary("AceAddon-2.0"):new("AceDB-2.0", "AceConsole-2.0", "AceComm-2.0", "FuBarPlugin-2.0")
local fmBuffCheck, self = fmBuffCheck, fmBuffCheck

fmBuffCheck.commPrefix = "fmbc"
fmBuffCheck.commDist = "GROUP"
fmBuffCheck.player = UnitName("player")

-- FuBar configuration
fmBuffCheck.hasIcon = true
fmBuffCheck.hideWithoutStandby = true
fmBuffCheck.defaultPosition = "MINIMAP"
fmBuffCheck.defaultMinimapPosition = 200
fmBuffCheck.hasNoText = true
fmBuffCheck.hasNoColor = true
fmBuffCheck.cannotDetachTooltip = true
fmBuffCheck.independentProfile = false
fmBuffCheck.blizzardTooltip = true

fmBuffCheck:RegisterDB("fmBuffCheckDB", "fmBuffCheckDBPC")
fmBuffCheck:RegisterDefaults("profile", { debugging = false, nagging = false} )

fmBuffCheck.options = {
	type = 'group',
	handler = fmBuffCheck,
	args = {
		debugMode = {
			order = 10,
			name = L["Debugging Mode"],
			desc = L["If enabled, additional messages are printed out to help debugging."],
			type = 'toggle',
			get = function() return self.db.profile.debugging end,
			set = function(v) self.db.profile.debugging = v end,
		},
		nagMode = {
			order = 20,
			name = L["Nagging Mode"],
			desc = L["If enabled, every person NOT having all buffs is being reminded by whisper."],
			type = 'toggle',
			get = function() return self.db.profile.nagging end,
			set = function(v) self.db.profile.nagging = v end,
		}
	}
}

fmBuffCheck.OnMenuRequest = fmBuffCheck.options

function fmBuffCheck:OnInitialize()
	self:SetCommPrefix(self.commPrefix)
	self:RegisterComm(self.commPrefix, self.commDist, "OnCommReceive")
	self:UpdateTooltip()
end

function fmBuffCheck:OnEnable()
	self:RegisterChatCommand('/fmBuffCheck', '/fmbc', {
		type = 'execute',
		func = function()
			self:Request()
		end
	})
end

function fmBuffCheck:Request()
	-- request an update of all buffs from group members
	self:DebugPrint(L["Requesting buff status of all party members..."])
	self.BuffState = {}
	self:Send("QUERY", "-", "-", "-", "-")
end

function fmBuffCheck:Send(cmd, offensive, defensive, weapon, food)
	self:SendCommMessage(self.commDist, cmd, offensive, defensive, weapon, food)
	self:OnCommReceive(self.commPrefix, self.player, self.commDist, cmd, offensive, defensive, weapon, food)
end

function fmBuffCheck:OnCommReceive(prefix, sender, distribution, cmd, offensive, defensive, weapon, food)
	-- receive buff status off members or command to update buffs
	if cmd == "QUERY"  then
		self:DebugPrint((L["Buff list request received from %s."]):format(sender))
		self:GetAndSendBuffs()
	elseif cmd == "REPORT" then
		if IsPartyLeader() or IsRaidLeader() or IsRaidOfficer() then
			self:Print(("%s: %s | %s | %s | %s"):format(sender, self:PP(offensive), self:PP(defensive), self:PP(weapon), self:PP(food)))
			self:Alert(sender, offensive, defensive, weapon, food)
		end
	else
		self:DebugPrint((L["Unknown QUERY request received from %s: %s"]):format(sender, cmd))
	end
end

function fmBuffCheck:GetAndSendBuffs()
	local buff =
	{
		["o"] = { ["name"] = "", 	["time"] = 0, 	["charges"] = 0 },
		["d"] =	{ ["name"] = "",	["time"] = 0,	["charges"] = 0	},
		["f"] =	{ ["name"] = "",	["time"] = 0,	["charges"] = 0	},
		["w"] =	{ ["name"] = "",	["time"] = 0,	["charges"] = 0	}
	}
	
	for num = 1, 40 do
		local id = GetPlayerBuff(num, "HELPFUL")
		if id > 0 then
			local type = self:BuffParse(id)
			if type == "k" then
				buff["o"] = {["name"] = GetPlayerBuffName(id), ["time"] = GetPlayerBuffTimeLeft(id), ["charges"] = GetPlayerBuffApplications(id) }
			end
			if type == "w" then
				buff["d"] = {["name"] = GetPlayerBuffName(id), ["time"] = GetPlayerBuffTimeLeft(id), ["charges"] = GetPlayerBuffApplications(id) }
			end
			if type == "f" then
				buff["f"] = {["name"] = GetPlayerBuffName(id), ["time"] = GetPlayerBuffTimeLeft(id), ["charges"] = GetPlayerBuffApplications(id) }
			end
			
		end
	end
	
	buff["w"] = self:GetMainHandBuff()

	self:Send("REPORT", buff["o"], buff["d"], buff["w"], buff["f"])
	
end

function fmBuffCheck:GetMainHandBuff()
	local hasMainHandEnchant, mainHandExpiration, mainHandCharges, hasOffHandEnchant, offHandExpiration, offHandCharges = GetWeaponEnchantInfo(); 
	
	if hasMainHandEnchant then
		return {["name"] = self:WeaponParse(), ["time"] = mainHandExpiration / 1000, ["charges"] = mainHandCharges}
	else
		return {["name"] = "", ["time"] = 0, ["charges"] = 0}
	end
end


function fmBuffCheck:BuffParse(id)
	fmBuffCheckTooltip:ClearLines()
	fmBuffCheckTooltip:SetUnitBuff(self.player, id)
	
	for i = 1, fmBuffCheckTooltip:NumLines() do
		local txt = getglobal("fmBuffCheckTooltipTextLeft" .. i):GetText()
		
		if string.find(txt, L["Battle Elixir"], 1, true) then
			return "k"
		end
		
		if string.find(txt, L["Guardian Elixir"], 1, true) then
			return "w"
		end
		
		if string.find(txt, L["Well Fed"], 1, true) then
			return "f"
		end
    end
    return "??"
end

function fmBuffCheck:WeaponParse()
	fmBuffCheckTooltip:ClearLines()
	
	local slotId, _ = GetInventorySlotInfo("MainHandSlot")
	fmBuffCheckTooltip:SetInventoryItem(self.player, slotId)
	
	--for i = 1, fmBuffCheckTooltip:NumLines() do
	--	local txt = getglobal("fmBuffCheckTooltipTextLeft" .. i):GetText()
	--	self:DebugPrint(i .. ": " .. txt)
    --end
	return getglobal("fmBuffCheckTooltipTextLeft1"):GetText()
end

function fmBuffCheck:PP(obj)
	local name = obj["name"]
	local timeleft = obj["time"]

	if (name == nil or name == "") then
		return string.format("|cffff0000%s|r", L["none"]) 
	end
		
	if timeleft > 60 then
		timeleft = math.floor(timeleft / 60) .. "m"
	else
		timeleft = math.floor(timeleft) .. "s"
	end
	
	return string.format("|cff00ff00%s (%s)|r", name, timeleft) 
end

function fmBuffCheck:Alert(s, o, d, w, f)
	if self.db.profile.nagging then
	
		local ka, wa, fo, we, st
	
		ka = (o["name"] == nil or o["name"] == "")
		wa = (d["name"] == nil or d["name"] == "")
		fo = (f["name"] == nil or f["name"] == "")
		we = (w["name"] == nil or w["name"] == "")
		st = ""
	
		if (ka or wa or fo or we) then
			if ka then
				st = st .. L["[offensive elixir]"]
			end
			if wa then
				st = st .. L["[defensive elixir]"]
			end
			if fo then
				st = st .. L["[food buff]"]
			end
			if we then
				st = st .. L["[weapon buff]"]
			end
			SendChatMessage(L["You are missing the following buffs: "] .. st, "WHISPER", nil, s);
		end
	end
end

function fmBuffCheck:DebugPrint(msg)
	if self.db.profile.debugging then
		self.Print(msg)
	end
end

function fmBuffCheck:OnTooltipUpdate()
	GameTooltip:AddLine("fmBuffCheck")
	GameTooltip:AddLine(" ")
	GameTooltip:AddDoubleLine("Nagging:", (self.db.profile.nagging and YES or NO), 1, 1, 0, 1, 1, 1)
	GameTooltip:AddDoubleLine("Debugging:", (self.db.profile.debugging and YES or NO), 1, 1, 0, 1, 1, 1)
	GameTooltip:AddLine(" ")
	GameTooltip:AddLine("|cffffff00" .. L["Click|r to initiate buff check."], 0.2, 1, 0.2)
end

function fmBuffCheck:OnClick()
	local shift, alt, ctrl = IsShiftKeyDown(), IsAltKeyDown(), IsControlKeyDown()
	if not shift and not alt and not ctrl then
		self:Request()
	end
end